﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Core.Models;
using Syncfusion.EJ2.Navigations;
using Syncfusion.EJ2.Spreadsheet;
using Syncfusion.XlsIO;
using System.IO;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Internal;
using Newtonsoft.Json;
using Syncfusion.XlsIO.Implementation;

namespace Core.Controllers
{
    public class HomeController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        public IActionResult Open(IFormCollection openRequest)
        {
            ExcelEngine excelEngine = new ExcelEngine();
            IWorkbook workbook;
            Stream memStream = (openRequest.Files[0] as IFormFile).OpenReadStream();
            workbook = excelEngine.Excel.Workbooks.Open(memStream, ExcelOpenType.Automatic);
            List<CustomArgs> customArgs = GetCustomArgs(workbook);
            MemoryStream outputStream = new MemoryStream();
            workbook.SaveAs(outputStream);
            IFormFile formFile = new FormFile(outputStream, 0, outputStream.Length, "", openRequest.Files[0].FileName); // converting MemoryStream to IFormFile
            OpenRequest open = new OpenRequest();
            open.File = formFile;
            var content = Workbook.Open(open);
            content = content.Insert(content.Length - 2, ", \"customArgs\":" + JsonConvert.SerializeObject(customArgs)); // Adding customArgs
            return Content(content);
        }

        private List<CustomArgs> GetCustomArgs(IWorkbook workbook)
        {
            List<CustomArgs> customArgs = new List<CustomArgs>();
            IWorksheet worksheet;
            int i = 0, cnt = workbook.Worksheets.Count;
            while (i < cnt)
            {
                worksheet = workbook.Worksheets[i];
                for (int j = 0; j < worksheet.UsedRange.LastRow; j++)
                {
                    for (int k = 0; k < worksheet.UsedRange.LastColumn; k++)
                    {
                        customArgs.Add(new CustomArgs { address = RangeImpl.GetCellName(k + 1, j + 1), customValue = "custom value for " + RangeImpl.GetCellName(k + 1, j + 1), sheetIndex = i });
                    }
                }
                i++;
            }
            return customArgs;
        }

        public IActionResult Save(SaveSettings saveSettings)
        {
            return Workbook.Save(saveSettings);
        }

        public IActionResult About()
        {
            ViewData["Message"] = "Your application description page.";

            return View();
        }

        public IActionResult Contact()
        {
            ViewData["Message"] = "Your contact page.";

            return View();
        }

        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }

    public class CustomArgs
    {
        public string customValue { get; set; }
        public string address { get; set; }
        public int sheetIndex { get; set; }
    }
}
